#!/usr/bin/ruby

# generate-symbols.rb -- code generator for literal symbols
# September 24, 2008
#
# Copyright (C) 2007-2008 Devin Papineau
#
# This software is provided 'as-is', without any express or implied
# warranty.  In no event will the authors be held liable for any damages
# arising from the use of this software.
#
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely, subject to the following restrictions:
#
# 1. The origin of this software must not be misrepresented; you must not
#    claim that you wrote the original software. If you use this software
#    in a product, an acknowledgment in the product documentation would be
#    appreciated but is not required.
# 2. Altered source versions must be plainly marked as such, and must not be
#    misrepresented as being the original software.
# 3. This notice may not be removed or altered from any source distribution.
#
# Devin Papineau jdmpapin@uwaterloo.ca

$CPP = 'cpp'

# these should do.
$esc = Hash.new { |h, k| k }.merge('q' => '?',
                                   'b' => '!',
                                   'p' => '+',
                                   'm' => '-',
                                   't' => '*',
                                   'd' => '/',
                                   'D' => '.',
                                   'r' => '%',
                                   'a' => '&',
                                   'o' => '|',
                                   'x' => '^',
                                   'n' => '~',
                                   'l' => '<',
                                   'g' => '>',
                                   'e' => '=',
                                   'c' => ':',
                                   'A' => '@',
                                   '_' => "\n")

def dollar_escape(str)
  str.gsub(/\$(.)/) { $esc[$1] }
end

out_file = $*.shift
out_path = out_file.split(/\//)
Dir.chdir out_path.shift while out_path.size > 1
out_file = out_path[0]

out_header = out_file.sub /\.c\Z/, '.h'

File.unlink out_header if test ?e, out_header
system "touch", out_header

all_syms = []

(Dir['*.[ch]'] - [out_file, out_header]).
  map { |f| `#$CPP #{$*.join(' ')} "#{f}"` }.join.
  scan(/\bSYM_(##)?([A-Za-z0-9_$]+)\b/) do
  all_syms << $2
end

all_syms = all_syms.sort.uniq

cout = open out_file, 'w'
hout = open out_header, 'w'

hout.puts <<"EOF"
#ifndef #{out_header.gsub /\W/, '_'}
#define #{out_header.gsub /\W/, '_'}
EOF

hout.puts '#include "value.h"'
hout.puts 'extern void ch_symLiterals_init(void);'
cout.puts "#include \"#{out_header}\""
cout.puts '#include "sym.h"'
cout.puts '#include "mem.h"'

cout.puts "void ch_symLiterals_init(void)"
cout.puts "{"

all_syms.each do |sym|
  hout.puts "extern ch_value_t SYM_#{sym};"
  cout.puts "\tch_globalRoot(&SYM_#{sym}.sym);"

  text = dollar_escape(sym)
  cstr = text.gsub(%r'"|\\', '\\\&')
  cstr.gsub!(/\n/, '\n')
  cout.puts "\tSYM_#{sym} = ch_internSizedString(#{text.size}, \"#{cstr}\");"
end

hout.puts "#endif"
cout.puts "}"

all_syms.each do |sym|
  cout.puts "ch_value_t SYM_#{sym} = { 0, { 0 } };"
end

cout.close
hout.close

if test ?e, out_header + '.bak'
  File.unlink out_header + '.bak'
end
