#!/bin/sh

# install.sh -- chai install script
# September 30, 2008
#
# Copyright (C) 2007-2008 Devin Papineau
#
# This software is provided 'as-is', without any express or implied
# warranty.  In no event will the authors be held liable for any damages
# arising from the use of this software.
#
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely, subject to the following restrictions:
#
# 1. The origin of this software must not be misrepresented; you must not
#    claim that you wrote the original software. If you use this software
#    in a product, an acknowledgment in the product documentation would be
#    appreciated but is not required.
# 2. Altered source versions must be plainly marked as such, and must not be
#    misrepresented as being the original software.
# 3. This notice may not be removed or altered from any source distribution.
#
# Devin Papineau jdmpapin@uwaterloo.ca

PREFIX="$1"
if [ "x$PREFIX" = x ]; then
    PREFIX=/usr/local
    if [ -f .default_prefix ]; then
        READ_PREFIX=$(head -n 1 .default_prefix)
        if [ "x$READ_PREFIX" != x ]; then
            PREFIX="$READ_PREFIX"
        fi
        unset READ_PREFIX
    fi
else
    shift
fi

cmd () {
    echo "$@"
    "$@"
}

echo "$PREFIX" > .installed_prefix

# Install binaries
if [ ! -d "$PREFIX/bin" ]; then
    cmd mkdir -p "$PREFIX/bin"
fi

cmd install -c chai "$PREFIX/bin"
cmd install -c repl.ch "$PREFIX/bin"

# Install libchai.so and the contents of lib/chai
if [ ! -d "$PREFIX/lib/chai/_prelude_priv" ]; then
    cmd mkdir -p "$PREFIX/lib/chai/_prelude_priv"
fi

LIB_EXT="$(./lib_ext.sh)"

cmd install -c "libchai$LIB_EXT" "$PREFIX/lib"
cmd install -c lib/chai/prelude.ch "$PREFIX/lib/chai"
cmd install -c lib/chai/sourcify.ch "$PREFIX/lib/chai"
cmd install -c lib/chai/iter.ch "$PREFIX/lib/chai"
cmd install -c lib/chai/send_ops.ch "$PREFIX/lib/chai"
cmd install -c libmath$LIB_EXT "$PREFIX/lib/chai/math$LIB_EXT"
cmd install -c libsys$LIB_EXT "$PREFIX/lib/chai/sys$LIB_EXT"

cmd install -c lib/chai/_prelude_priv/Dict.ch "$PREFIX/lib/chai/_prelude_priv"
cmd install -c lib/chai/_prelude_priv/List.ch "$PREFIX/lib/chai/_prelude_priv"
cmd install -c lib/chai/_prelude_priv/_prelude_init.ch "$PREFIX/lib/chai/_prelude_priv"
cmd install -c lib/chai/_prelude_priv/_prelude_init2.ch "$PREFIX/lib/chai/_prelude_priv"
cmd install -c lib/chai/_prelude_priv/_prelude_iterable_extra.ch "$PREFIX/lib/chai/_prelude_priv"
cmd install -c lib/chai/_prelude_priv/macros.ch "$PREFIX/lib/chai/_prelude_priv"

# Install headers
if [ ! -d "$PREFIX/include/chai" ]; then
    cmd mkdir -p "$PREFIX/include/chai"
fi

for HEADER in src/*.h; do
    if [ "$HEADER" = src/sym_literals.h ]; then
        echo skipping sym_literals.h
    else
        cmd install -c "$HEADER" "$PREFIX/include/chai"
    fi
done
