#!/usr/bin/env chai
;;;; -*- mode: chai -*-

;; Dict.ch -- prelude-defined Dict methods
;; September 24, 2008
;;
;; Copyright (C) 2007-2008 Devin Papineau
;;
;; This software is provided 'as-is', without any express or implied
;; warranty.  In no event will the authors be held liable for any damages
;; arising from the use of this software.
;;
;; Permission is granted to anyone to use this software for any purpose,
;; including commercial applications, and to alter it and redistribute it
;; freely, subject to the following restrictions:
;;
;; 1. The origin of this software must not be misrepresented; you must not
;;    claim that you wrote the original software. If you use this software
;;    in a product, an acknowledgment in the product documentation would be
;;    appreciated but is not required.
;; 2. Altered source versions must be plainly marked as such, and must not be
;;    misrepresented as being the original software.
;; 3. This notice may not be removed or altered from any source distribution.
;;
;; Devin Papineau jdmpapin@uwaterloo.ca

class: Dict {
   ;;; Equality is not properly defined in the interpreter.
   def: #|==| (d) {
       if: .eq?(d) { true }
       :: not d.is_a?(Dict) or .size != d.size { false }
       :: else {
           my [ks, i, len] = [.keys, -1, .size]
           leave: {
               while: (i += 1) < len {
                   my k = ks(i)
                   if: !d.has_key?(k) or self(k) != d(k) { return(false) }
               }
               true
           }
       }
   }

   ;;; Hash value must be equal when dicts are equal
   def: hash {
       my hash = 0
       ;; can't guarantee keys are in any given order. Just use a sum
       for: k in .keys {
           hash += k.hash + self(k).hash * 11
       }
       hash
   }
}
