#!/usr/bin/env chai
;;;; -*- mode: chai -*-

;; List.ch -- prelude-defined List methods
;; September 24, 2008
;;
;; Copyright (C) 2007-2008 Devin Papineau
;;
;; This software is provided 'as-is', without any express or implied
;; warranty.  In no event will the authors be held liable for any damages
;; arising from the use of this software.
;;
;; Permission is granted to anyone to use this software for any purpose,
;; including commercial applications, and to alter it and redistribute it
;; freely, subject to the following restrictions:
;;
;; 1. The origin of this software must not be misrepresented; you must not
;;    claim that you wrote the original software. If you use this software
;;    in a product, an acknowledgment in the product documentation would be
;;    appreciated but is not required.
;; 2. Altered source versions must be plainly marked as such, and must not be
;;    misrepresented as being the original software.
;; 3. This notice may not be removed or altered from any source distribution.
;;
;; Devin Papineau jdmpapin@uwaterloo.ca

class: List {
   ;;; Equality is not properly defined in the interpreter.
   def: #|==| (xs) {
       if: .eq?(xs) { true }
       :: not xs.is_a?(List) or .size != xs.size { false }
       :: else {
           my [i, len] = [-1, .size]
           leave: {
               while: (i += 1) < len {
                   if: self(i) != xs(i) { return(false) }
               }
               true
           }
       }
   }

   ;;; Hashes always must be equal when lists are
   def: hash {
       my hash = 0
       for: x in self {
           hash *= 17
           hash += x.hash
       }
       hash
   }

   ;;; Find the index of an element
   def: index(x) {
       leave: {
           for: i in .size { if: x == self(i), return(i) }
       }
   }
}
