#!/usr/bin/env chai
;;;; -*- mode: chai -*-

;; macros.ch -- prelude-defined macros
;; September 24, 2008
;;
;; Copyright (C) 2007-2008 Devin Papineau
;;
;; This software is provided 'as-is', without any express or implied
;; warranty.  In no event will the authors be held liable for any damages
;; arising from the use of this software.
;;
;; Permission is granted to anyone to use this software for any purpose,
;; including commercial applications, and to alter it and redistribute it
;; freely, subject to the following restrictions:
;;
;; 1. The origin of this software must not be misrepresented; you must not
;;    claim that you wrote the original software. If you use this software
;;    in a product, an acknowledgment in the product documentation would be
;;    appreciated but is not required.
;; 2. Altered source versions must be plainly marked as such, and must not be
;;    misrepresented as being the original software.
;; 3. This notice may not be removed or altered from any source distribution.
;;
;; Devin Papineau jdmpapin@uwaterloo.ca

;;; Scoped closing files
defmacro: (withfile: assign, mode, body = nil) {
    if: not body {
        body = mode
        mode = "r"
    }

    if: !assign.is_a?(List) or assign(0) != #"op_=" or
        !assign(1).is_a?(Symbol)
    {
        raise ParseError("withfile{} expects an assignment to an identifier")
    }

    my [f, filename] = assign.slice(1, -1)
    if: mode.is_a?(Symbol) { mode = mode.to_s }

    ~~ _builtins:with_file_func(~.filename, ~.mode, fn: ~.f { ~.body })
}

def: with_file_func(name, mode, thunk) {
    my result
    my f = File(name, mode)

    try: { result = thunk(f) }
    :: ensure {
        f.close
        result
    }
}
;;; ----


;;; while with a name for the result of the condition
defmacro: (whilet: condition, body) {
    my name = #it

    if: condition.is_a?(List) and condition(0) == #"op_=" and
        condition(1).is_a?(Symbol)
    {
        name = condition(1)
        condition = condition(2)
    }

    ~~ let: ~.name {
        while: ~.name = ~.condition { ~.body }
    }
}
;;;; ----


;;;; Do something unconditionally, with a conditional before,
;;;; and a conditional after on the same condition
defmacro: (wrapif: test, then1, always, then2) {
    my cond = gensym()

    ~~ let: ~.cond = ~.test {
        if: ~.cond { ~.then1 }
        ~.always
        if: ~.cond { ~.then2 }
    }
}
;;;; ----


;;;; Do something a certain number of times
defmacro: (dotimes: var, n, body = nil) {
    if: not body {
        body = n
        n = var
        var = gensym()
    }

    my lim = gensym()

    ~~ let: ~.var = 0, ~.lim = ~.n {
        while: ~.var < ~.lim {
            ~.body
            ~.var += 1
        }
    }
}
;;;; ----
