#!/usr/bin/chai
;;; -*- mode: chai -*-

;; iter.ch -- chai iter module. map/reduce/etc.
;; October 3, 2008
;;
;; Copyright (C) 2007-2008 Devin Papineau
;;
;; This software is provided 'as-is', without any express or implied
;; warranty.  In no event will the authors be held liable for any damages
;; arising from the use of this software.
;;
;; Permission is granted to anyone to use this software for any purpose,
;; including commercial applications, and to alter it and redistribute it
;; freely, subject to the following restrictions:
;;
;; 1. The origin of this software must not be misrepresented; you must not
;;    claim that you wrote the original software. If you use this software
;;    in a product, an acknowledgment in the product documentation would be
;;    appreciated but is not required.
;; 2. Altered source versions must be plainly marked as such, and must not be
;;    misrepresented as being the original software.
;; 3. This notice may not be removed or altered from any source distribution.
;;
;; Devin Papineau jdmpapin@uwaterloo.ca

def: each(f, xs) {
    for: x in xs { f(x) }
}

def: map(f, xs) {
    iter: {
        my next = xs.iter
        fn: { f(next()) }
    }
}

def: catmap(f, xs) {
    iter: {
        my next = xs.iter
        my next_from_f = [].iter

        fn: {
            my done
            my result
            while: not done {
                done = true
                try: {
                    result = next_from_f()
                } :: StopIterationException {
                    next_from_f = f(next()).iter
                    done = false
                }
            }
            result
        }
    }
}

def: select(f, xs) {
    iter: {
        my next = xs.iter
        fn: {
            my r
            while: not f(r = next()) { }
            r
        }
    }
}

def: reject(f, xs) { xs.select(fnot(f)) }

def: take(n, xs) {
    my result = []
    my next = xs.iter
    for: x in 1..n { result.push(next()) }
    result
}

def: drop(n, xs) {
    iter: {
        my next = xs.iter
        dotimes: i, n { next() }
        next
    }
}

def: zip(xs, ys, *zss) {
    iter: {
        my next = xs.iter
        my nexty = ys.iter
        my nextzs = list(map(#iter, zss))

        fn: {
            my r = [next(), nexty()]
            for: f in nextzs { r.push(f()) }
            r
        }
    }
}

def: reduce(z, f, xs) {
    for: x in xs { z = f(z, x) }
    z
}

def: any?(f, xs) {
    leave: {
        for: x in xs { f(x) and return(true) }
        false
    }
}

def: all?(f, xs) {
    leave: {
        for: x in xs { f(x) or return(false) }
        true
    }
}

def: member?(value, xs) {
    any?(xs, x -> value == x)
}

def: list(xs) {
    my result = []
    for: x in xs { result.push(x) }
    result
}

def: zipmap(f, xs) { zip(xs, map(xs, f)) }

def: join(sep, xs) {
    my r = ""

    my first = true
    for: x in xs {
        if: first { first = false } :: { r.concat!(sep) }
        r.concat!(x)
    }

    r
}

def: find(f, xs) {
    leave: {
        for: x in xs { if: f(x) { return(x) } }
    }
}

def: iter(startval, next) {
    iter: {
        my x = startval
        fn: {
            my r = x
            x = next(x)
            r
        }
    }
}

def: cat(*xss) {
    catmap(xs -> xs, xss)
}
