#!/usr/bin/env chai
;; -*- mode: chai -*-

;; repl.ch -- chai interactive read-eval-print loop
;; September 30, 2008
;;
;; Copyright (C) 2007-2008 Devin Papineau
;;
;; This software is provided 'as-is', without any express or implied
;; warranty.  In no event will the authors be held liable for any damages
;; arising from the use of this software.
;;
;; Permission is granted to anyone to use this software for any purpose,
;; including commercial applications, and to alter it and redistribute it
;; freely, subject to the following restrictions:
;;
;; 1. The origin of this software must not be misrepresented; you must not
;;    claim that you wrote the original software. If you use this software
;;    in a product, an acknowledgment in the product documentation would be
;;    appreciated but is not required.
;; 2. Altered source versions must be plainly marked as such, and must not be
;;    misrepresented as being the original software.
;; 3. This notice may not be removed or altered from any source distribution.
;;
;; Devin Papineau jdmpapin@uwaterloo.ca

import: sys

class: InterruptException, Exception { }
sys:signal(#int, fn: signal {
    raise InterruptException()
})

def: keep_reading?(line, stack) {
    leave: {
        my string? = false
        my escape? = false

        for: c in line.chars {
            if: string? {
                if: escape? {
                    escape? = false
                } :: c == "\\" {
                    escape? = true
                } :: c == "\"" {
                    string? = false
                }
            } :: c == "\"" {
                string? = true
            } :: c == "[" or c == "(" or c == "{" {
                stack.push(c)
            } :: c == "]" and stack.pop != "[" or
                 c == ")" and stack.pop != "(" or
                 c == "}" and stack.pop != "{" {
                return(false)
            }
        }

        not stack.empty?
    }
}

def: run_repl() {
    leave: {
        my expr = ""
        my stack = []

        while: true {
            try: {
                while: true {
                    print(if: expr.empty?, "chai> ", "... > ")

                    my line = gets()
                    if: not line { return() }

                    try: {
                        expr = expr ++ line
                        if: not keep_reading?(line, stack) {
                            my e = expr
                            expr = ""
                            stack = []
                            my result = eval(e)
                            if: not result.nil? {
                                puts("=> ", result.pretty_print(3).substr(3, -1))
                            }
                        }
                    } :: Exception => e {
                        puts(e.pretty_error)
                    }
                }
            } :: InterruptException {
                ;; start reading over again
                puts()
                expr = ""
                stack = []
            }
        }
    }
}

run_repl()
