/* Class.c -- implementation of chai Class class
   September 24, 2008

  Copyright (C) 2007-2008 Devin Papineau

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

  Devin Papineau jdmpapin@uwaterloo.ca
*/

#include "assorted.h"
#include "obj.h"
#include "value.h"
#include "Exception.h"
#include "sym_literals.h"
#include <stdio.h>

static ch_value_t class_init(ch_value_t self, long argc, ch_value_t* args)
{
    ch_value_t name = kNil;
    ch_value_t superclass = ch_cObject;

    ch_checkArgs(argc, args, 0, 2, 0, &superclass, &name);

    if(CH_FALSEP(ch_isa(superclass, ch_cClass)))
        ch_raise2(ch_eTypeError, "can't inherit from non-Class object");

    ch_class_t* class = (ch_class_t*)self.obj->data;
    class->superclass = superclass.obj;
    class->reprID = ((ch_class_t*)superclass.obj->data)->reprID;

    ch_setSlot(self, SYM_name, name);

    /* Maintain list of subclasses */
    CH_SEND(((ch_class_t*)class->superclass->data)->subclasses, push, self);

    return kNil;
}

static ch_value_t class_new(ch_value_t self, long argc, ch_value_t* args)
{
    ch_value_t inst = ch_obj_create(self);
    ch_vsend(inst, SYM__init_, argc, args);
    return inst;
}

static ch_value_t class_toString(ch_value_t self)
{
    ch_value_t name;
    if(ch_getSlot(self, SYM_name, &name) < 0)
        name = ch_string_new1("[unnamed]");

    if(name.type == CH_NIL) {
        ch_obj_t* myClass = self.obj->class;
        self.obj->class = ch_cObject.obj;
        ch_value_t str = CH_SEND(self, to_s);
        self.obj->class = myClass;
        return str;
    }

    return CH_SEND(name, to_s);
}

static ch_value_t class_defineMethod(ch_value_t self,
                                     ch_value_t name,
                                     ch_value_t func)
{
    ch_defineMethodRaw(self, name, func);
    return name;
}

static ch_value_t class_method_implementation_unsafe(ch_value_t self,
                                                     ch_value_t name)
{
    ch_class_t *klass = (ch_class_t*)self.obj->data;
    ch_value_t result;

    if (name.type != CH_SYMBOL) {
        ch_value_t as_str = CH_SEND(name, inspect);
        if (CH_FALSEP(ch_isa(as_str, ch_cString)))
            ch_raise2(ch_eTypeError, "expected symbol method name");
        else
            ch_raise2(ch_eTypeError, "expected symbol method name, got %s",
                      CH_CSTR(as_str));
    }

    ch_ht_get(&klass->methods, name, &result);
    if (result.type == CH_NIL)
        return result;

    if (result.ref->type == CH_REF)
        result = *result.ref;

    return *result.ref;
}

static ch_value_t class_superclass(ch_value_t self)
{
    ch_class_t *klass = (ch_class_t*)self.obj->data;
    ch_value_t r;

    if (!klass->superclass)
        return kNil;

    r.type = CH_OBJECT;
    r.obj = klass->superclass;
    return r;
}

void ch_defineClassMethods()
{
    ch_defineMethod(ch_cClass, SYM__init_, -2, (ch_cfn_t)class_init);
    ch_defineMethod(ch_cClass, SYM__call_, -2, (ch_cfn_t)class_new);
    ch_defineMethod(ch_cClass, SYM_new, -2, (ch_cfn_t)class_new);
    ch_defineMethod(ch_cClass, SYM_superclass, 0, (ch_cfn_t)class_superclass);
    ch_defineMethod(ch_cClass,
                    SYM_to_s,
                    0,
                    (ch_cfn_t)class_toString);
    ch_defineMethod(ch_cClass,
                    SYM_inspect_raw,
                    0,
                    (ch_cfn_t)class_toString);
    ch_defineMethod(ch_cClass,
                    SYM_define_method,
                    2,
                    (ch_cfn_t)class_defineMethod);

    ch_defineMethod(ch_cClass,
                    SYM_method_implementation_unsafe,
                    1,
                    (ch_cfn_t)class_method_implementation_unsafe);
}
