/* Exception.c -- chai error handling/Exception interface
   September 30, 2008

  Copyright (C) 2007-2008 Devin Papineau

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

  Devin Papineau jdmpapin@uwaterloo.ca
*/

#ifndef CH_EXCEPTION_H
#define CH_EXCEPTION_H

#include "value.h"
#include <stddef.h>
#include <setjmp.h>

enum {
    CH_PROTECT_NORMAL = 0,
    CH_PROTECT_UNWOUND = 1
};

void ch_handlers_init(void);
ch_value_t ch_protect(ch_value_t (*f)(ch_value_t), ch_value_t arg, int* status);
__attribute__((noreturn)) void ch_unwind(void);
__attribute__((noreturn)) void ch_raise(ch_value_t value);
__attribute__((noreturn)) void ch_raise2(ch_value_t type, const char *fmt, ...);
void ch_Exceptions_init(void);

extern ch_value_t ch_eException;
extern ch_value_t ch_eTypeError;
extern ch_value_t ch_eNameError;
extern ch_value_t ch_eArgumentError;
extern ch_value_t ch_eNoMethodError;
extern ch_value_t ch_eUnimplementedError;
extern ch_value_t ch_eBytecodeFormatError;
extern ch_value_t ch_eFileModeError;
extern ch_value_t ch_eOutOfBoundsError;
extern ch_value_t ch_eArityError;
extern ch_value_t ch_eParseError;
extern ch_value_t ch_eStackDepthError;
extern ch_value_t ch_eFileOpenError;
extern ch_value_t ch_eDivideByZeroError;

/* Bytecode.c likes to roll its own */
extern size_t ch_handlers_size;
extern size_t ch_handlers_top;
extern jmp_buf *ch_handlers;

#endif /* CH_EXCEPTION_H */
