/* Module.h -- chai module interface
   October 3, 2008

  Copyright (C) 2007-2008 Devin Papineau

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

  Devin Papineau jdmpapin@uwaterloo.ca
*/

#ifndef MODULE_H
#define MODULE_H

#include "value.h"
#include "obj.h"

typedef struct {
    ch_ht_t globals;
    ch_ht_t macros;
    ch_ht_t imports;
    ch_ht_t imported_names;
    ch_ht_t prec_table;
    ch_ht_t uprec_table;
    ch_ht_t assoc_table;
    ch_ht_t op_idents;
    ch_value_t module_name;
} ch_module_t;

extern ch_value_t ch_cModule, ch_current_module;

#define ch_bind_global(mod, name, var)           \
    ch_bind_global_((mod), SYM_##name, &(var))

void ch_Module_init(void);
ch_value_t ch_load_module(const char *name);
void ch_lookup_global_name(ch_value_t m, ch_value_t name,
                           ch_value_t *rm, ch_value_t *rname);
void ch_lookup_macro_name(ch_value_t m, ch_value_t name,
                          ch_value_t *rm, ch_value_t *rname);
// bind_global returns the ref
ch_value_t ch_bind_global_(ch_value_t mod, ch_value_t name, ch_value_t *place);
void ch_set_global(ch_value_t mod, ch_value_t name, ch_value_t val);
ch_value_t ch_get_global(ch_value_t mod, ch_value_t name);
void ch_module_import(ch_value_t mod, ch_value_t imp, ch_value_t name);
void ch_module_import_name_from(ch_value_t m, ch_value_t name, ch_value_t from);
void ch_module_import_all_from(ch_value_t m, ch_value_t from);
void ch_module_import_op_from(ch_value_t m, ch_value_t op, ch_value_t from);
void ch_module_import_uop_from(ch_value_t m, ch_value_t op, ch_value_t from);
void ch_module_import_all_ops_from(ch_value_t m, ch_value_t from);
ch_value_t ch_module_precedence_of(ch_value_t m, ch_value_t op,
                                   ch_value_t unaryp);
ch_value_t ch_get_module(ch_value_t name);
void ch_inject_module(ch_value_t m);

static inline ch_value_t ch_current_module_name(void)
{
    return ((ch_module_t*)ch_current_module.obj->data)->module_name;
}

#define ch_defvar(k, v) ch_bind_global_(ch_current_module, SYM_##k, &(v))
#define ch_defn(name, arity, cfn)                     \
    do {                                              \
        static ch_value_t _0f;                        \
        _0f = ch_prim_new((arity), (ch_cfn_t)(cfn));  \
        ch_setSlot(_0f, SYM_name,                     \
                   ch_string_new1(#name));            \
        ch_setSlot(_0f, SYM_containing_module,        \
                   ch_current_module_name());         \
        ch_defvar(name, _0f);                         \
    } while (0)

#endif /* MODULE_H */
