/* Range.c -- implementation of chai Range class
   September 24, 2008

  Copyright (C) 2007-2008 Devin Papineau

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

  Devin Papineau jdmpapin@uwaterloo.ca
*/

#include "assorted.h"
#include "sym.h"
#include "sym_literals.h"
#include "obj.h"

ch_value_t ch_cRange = { 0, { 0 } };
ch_value_t ch_cIntRange = { 0, { 0 } };

static ch_value_t rng_init(ch_value_t self, ch_value_t lo, ch_value_t hi)
{
    ch_setSlot(self, SYM_lo, lo);
    ch_setSlot(self, SYM_hi, hi);

    return kNil;
}

static ch_value_t rng_each(ch_value_t self, ch_value_t f)
{
    ch_value_t i;
    ch_value_t end;

    ch_getSlot(self, SYM_lo, &i);
    ch_getSlot(self, SYM_hi, &end);

    for (; CH_TRUEP(CH_SEND(i, $l$e, end)); i = CH_SEND(i, succ))
        ch_vfuncall(f, 1, &i);

    return kNil;
}

static ch_value_t intrng_each(ch_value_t self, ch_value_t f)
{
    ch_value_t i;
    ch_value_t end;

    ch_getSlot(self, SYM_lo, &i);
    ch_getSlot(self, SYM_hi, &end);

    if (i.type != CH_INT || end.type != CH_INT)
        return rng_each(self, f);

    for (; i.i <= end.i; i.i++)
        ch_vfuncall(f, 1, &i);

    return kNil;
}

static ch_value_t rng_toList(ch_value_t self)
{
    ch_value_t result = ch_list_new();

    ch_value_t i;
    ch_value_t end;

    ch_getSlot(self, SYM_lo, &i);
    ch_getSlot(self, SYM_hi, &end);

    for (; CH_TRUEP(CH_SEND(i, $l$e, end)); i = CH_SEND(i, succ))
        CH_SEND(result, push, i);

    return result;
}

void ch_Range_init()
{
    ch_globalRoot(&ch_cRange.obj);
    ch_globalRoot(&ch_cIntRange.obj);
    ch_cRange = ch_class_create(SYM_Range, ch_cObject);
    ch_cIntRange = ch_class_create(SYM_IntRange, ch_cRange);

    ch_defineMethod(ch_cRange, SYM__init_, 2, (ch_cfn_t)rng_init);
    ch_defineMethod(ch_cRange, SYM_each, 1, (ch_cfn_t)rng_each);
    ch_defineMethod(ch_cRange, SYM_to_l, 0, (ch_cfn_t)rng_toList);

    ch_defineMethod(ch_cIntRange, SYM_each, 1, (ch_cfn_t)intrng_each);
}
