/* Symbol.c -- implementation of chai Symbol class
   September 24, 2008

  Copyright (C) 2007-2008 Devin Papineau

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

  Devin Papineau jdmpapin@uwaterloo.ca
*/

#include "assorted.h"
#include "obj.h"
#include "value.h"
#include "sym_literals.h"

static ch_value_t sym_toString(ch_value_t self)
{
    return ch_string_new1(&self.sym[sizeof(long)]);
}

static ch_value_t sym_eq(ch_value_t self, ch_value_t b)
{
    return (self.type == b.type && self.sym == b.sym) ? kTrue : kFalse;
}

static ch_value_t sym_hash(ch_value_t self)
{
    return ch_fix(*((long*)self.sym));
}

static ch_value_t sym_inspect(ch_value_t self, ch_value_t b)
{
    ch_value_t result = ch_string_new1("#");
    ch_value_t str = sym_toString(self);

    char *cstr = CH_CSTR(str);
    int i, quote_flag = 0;

    for (i = 0; cstr[i]; i++) {
        if ((cstr[i] < 'a' || cstr[i] > 'z') && (cstr[i] < 'A' || cstr[i] > 'Z')
            && (cstr[i] < '0' || cstr[i] > '9') && cstr[i] != '_'
            && (cstr[i+1] || (cstr[i] != '!' && cstr[i] != '?'))
            && (cstr[i] != ':' || i == 0 || cstr[i - 1] == ':')) {
            quote_flag = 1;
            break;
        }
    }

    if (quote_flag)
        CH_SEND(result, concat$b, CH_SEND2(str, inspect));
    else
        CH_SEND(result, concat$b, str);

    return result;
}

static ch_value_t sym_call(ch_value_t self, long argc, ch_value_t* args)
{
    if (argc < 1)
        ch_raise2(ch_eArgumentError, "expected at least 1 argument");

    return ch_vsend(args[0], self, argc - 1, &args[1]);
}

void ch_defineSymbolMethods()
{
    ch_defineMethod(ch_cSymbol, SYM_$e$e, 1, (ch_cfn_t)sym_eq);
    ch_defineMethod(ch_cSymbol, SYM_hash, 0, (ch_cfn_t)sym_hash);
    ch_defineMethod(ch_cSymbol, SYM__call_, -2, (ch_cfn_t)sym_call);
    ch_defineMethod(ch_cSymbol,
                    SYM_to_s,
                    0,
                    (ch_cfn_t)sym_toString);
    ch_defineMethod(ch_cSymbol, SYM_inspect_raw, 0, (ch_cfn_t)sym_inspect);
}
