/* assorted.h -- Random functions that didn't warrant their own headers
   September 24, 2008

  Copyright (C) 2007-2008 Devin Papineau

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

  Devin Papineau jdmpapin@uwaterloo.ca
*/

#ifndef CH_ASSORTED_H
#define CH_ASSORTED_H

#include "value.h"
#include "obj.h"
#include <inttypes.h>
#include <stdio.h>

/* variables */
extern ch_value_t ch_cRange, ch_cIntRange;
extern ch_value_t ch_cRegex, ch_cRegexMatchData, ch_eRegexCompilationError;

/* Closure functions */
extern void ch_Closure_init(void);
extern ch_value_t ch_closure_new(int ndefs, int env_size,
                                 ch_value_t bc, uint32_t ip);
extern void ch_defineClosureMethods(void);
extern void ch_callClosure(ch_value_t clo);

/* File functions */
extern void ch_File_init(void);

/* This expects an OPEN file pointer, and accurate info.  A NULL
 * filename only means that #inspect will use the address instead of
 * the filename. Closing the file becomes the responsibility of the
 * resulting object. */
extern ch_value_t ch_file_new(FILE* fp,
                              int reading,
                              int writing,
                              const char* filename);

/* Assorted class initialization functions */
extern void ch_defineBooleanMethods(void);
extern void ch_defineClassMethods(void);
extern void ch_defineDictMethods(void);
extern void ch_defineFloatMethods(void);
extern void ch_defineIntegerMethods(void);
extern void ch_defineListMethods(void);
extern void ch_defineNilClassMethods(void);
extern void ch_defineNumberMethods(void);
extern void ch_defineObjectMethods(void);
extern void ch_Range_init(void);
extern void ch_Regex_init(void);
extern void ch_defineStringMethods(void);
extern void ch_defineSymbolMethods(void);

#endif /* CH_ASSORTED_H */
