/* ht.h -- chai hash table interface
   September 24, 2008

  Copyright (C) 2007-2008 Devin Papineau

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

  Devin Papineau jdmpapin@uwaterloo.ca
*/

#ifndef CH_HT_H
#define CH_HT_H

#include "value.h"
#include <stddef.h>

typedef size_t (*ch_hashFunction_t)(ch_value_t);
typedef int (*ch_eqFunction_t)(ch_value_t,ch_value_t);

typedef struct ch_hashBucket {
    ch_value_t key;
    ch_value_t value;
    struct ch_hashBucket* next;
} ch_hashBucket_t;

typedef struct ch_hashTable {
    size_t cap;
    size_t mask;
    size_t size;
    ch_hashFunction_t hfn;
    ch_eqFunction_t eq;
    struct ch_hashBucket** buckets;
} ch_ht_t;

/* For hfn NULL means &ch_hashSym, and for eq it means &ch_shallowEq */
extern void ch_ht_init(ch_ht_t* ht,
                       ch_hashFunction_t hfn,
                       ch_eqFunction_t eq);
extern void ch_ht_scan(void* ptr);
extern ch_ht_t* ch_ht_create(ch_hashFunction_t hfn, ch_eqFunction_t eq);
extern void ch_ht_mergeb(ch_ht_t* ht, ch_ht_t* from);

/* Rehashing */
extern void ch_ht_rehash(ch_ht_t* ht, size_t newCap);

/* Predicates */
extern int ch_ht_hasKeyp(ch_ht_t* ht, ch_value_t key);
extern int ch_ht_emptyp(ch_ht_t* ht);

/* Things to actually DO with a hash table. */
extern void ch_ht_clear(ch_ht_t* ht);
extern void ch_ht_set(ch_ht_t* ht, ch_value_t key, ch_value_t value);
extern int ch_ht_get(ch_ht_t* ht, ch_value_t key, ch_value_t* result);
extern int ch_ht_delete(ch_ht_t* ht, ch_value_t key);
extern ch_value_t ch_ht_keys(ch_ht_t *ht);

/* This is nice to have around.
 * Pass negative len for NUL-terminated string. */
extern size_t ch_hashCString(const char* s, int len);

#endif /* CH_HT_H */
