/* main.c -- chai main program
   October 3, 2008

  Copyright (C) 2007-2008 Devin Papineau

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

  Devin Papineau jdmpapin@uwaterloo.ca
*/

#include "obj.h"
#include "sym.h"
#include "value.h"
#include "state.h"
#include "Bytecode.h"
#include "Exception.h"
#include "Module.h"
#include "compile.h"
#include <stdio.h>
#include <limits.h>

static int g_argc;
static char **g_argv;

static ch_value_t protect_main(ch_value_t ignore_)
{
    char *filename;
    ch_value_t bytecode, main_module_val;
    ch_module_t *main_module;
    int i;

    filename = "-"; /* stdin */
    if (g_argc > 1)
        filename = g_argv[1];

    ch_load("prelude");

    ch_argv = ch_list_new();
    for (i = 2; i < g_argc; i++)
        ch_send(ch_argv, ch_internCString("push"), 1,
                ch_string_new1(g_argv[i]));

    /* Switch to main module. */
    main_module_val = ch_obj_create(ch_cModule);
    main_module = (ch_module_t*)main_module_val.obj->data;
    main_module->module_name = ch_internCString("_main");
    ch_module_import(main_module_val, ch_m_builtins,
                     ch_internCString("_builtins"));
    ch_module_import_all_from(main_module_val, ch_m_builtins);
    ch_module_import_all_ops_from(main_module_val, ch_m_builtins);

    ch_module_import(main_module_val, ch_m_root, ch_internCString("_r"));
    ch_module_import(ch_m_root, main_module_val, ch_internCString("_main"));

    ch_current_module = main_module_val;
    ch_inject_module(main_module_val);

    if (!strcmp("-", filename))
        bytecode = ch_compile_stdio_file(stdin);
    else
        bytecode = ch_compile_file(filename);

    ch_runBytecode(bytecode);
    ch_pop();

    return ch_fix(0);
}

static ch_value_t protect_print_error(ch_value_t error)
{
    ch_value_t message, method;

    if (CH_FALSEP(ch_isa(error, ch_eException)))
        method = ch_internCString("inspect");
    else
        method = ch_internCString("pretty_error");

    message = ch_vsend(error, method, 0, NULL);
    fprintf(stderr, "%s%s", CH_CSTR(message),
            method.sym == ch_internCString("inspect").sym ? "\n" : "");
    return kNil;
}

int mem_init_main(int argc, char **argv)
{
    ch_symTable_init();
    ch_reprData_init();
    ch_state_init(argc > 1 ? argv[1] : "-");
    ch_compiler_init();

    g_argc = argc;
    g_argv = argv;


    // should maybe use the return value of ch_protect here?
    int status;
    ch_protect(protect_main, kNil, &status);
    if (status == CH_PROTECT_UNWOUND) {
        ch_value_t exc;

        exc = ch_get_global(ch_m_builtins, ch_internCString("\nexc"));
        ch_protect(protect_print_error, exc, &status);
        if (status == CH_PROTECT_UNWOUND) {
            fprintf(stderr,
                    "*** exception thrown while trying to "
                    "print exception. bailing ***\n");
        }

        return 1;
    }

    return 0;
}

int main(int argc, char** argv)
{
    uintptr_t i;
    ch_mem_init(&i);
    return mem_init_main(argc, argv);
}
