/* math.c -- implementation of the chai math module
   September 30, 2008

  Copyright (C) 2007-2008 Devin Papineau

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

  Devin Papineau jdmpapin@uwaterloo.ca
*/

#include <math.h>
#include "chai.h"

#define FV(x)                                                           \
    (x).type == CH_FLOAT ? (x).f                                        \
        : (x).type == CH_INT ? (float)((x).i)                           \
        : ({                                                            \
               ch_raise2(ch_eTypeError, "not a number");               \
               0.0;                                                    \
           })

#define DEF(name, val)                                                  \
    ch_set_global(ch_current_module, ch_internCString((#name)), (val))

#define DEFN(name, arity, fn)                   \
    defn_((#name), (arity), (ch_cfn_t)(fn))

static void defn_(const char *name, int arity, ch_cfn_t fn)
{
    ch_value_t impl = ch_prim_new(arity, fn);
    ch_value_t name_sym = ch_internCString(name);

    ch_setSlot(impl, ch_internCString("name"), name_sym);
    ch_setSlot(impl, ch_internCString("containing_module"),
               ch_current_module_name());
    ch_set_global(ch_current_module, name_sym, impl);
}

/* symbols */
static ch_value_t SYM_$l, SYM_$t, SYM_method_missing;

static ch_value_t ch_abs(ch_value_t x)
{
    if (CH_FALSEP(CH_SEND(x, $l, ch_fix(0))))
        return x;

    return CH_SEND(x, $t, ch_fix(-1));
}

/* thin C math function wrapper */
#define WRAP(name)                              \
    static ch_value_t ch_##name(ch_value_t x)   \
    {                                           \
        return ch_float(name(FV(x)));           \
    }

WRAP(sqrt);
WRAP(log);
WRAP(log2);
WRAP(log10);
WRAP(sin);
WRAP(cos);
WRAP(tan);
WRAP(asin);
WRAP(acos);
WRAP(atan);

/* initialize the module */
void chai_load_module(void)
{
    SYM_$l = ch_internCString("<");
    SYM_$t = ch_internCString("*");
    SYM_method_missing = ch_internCString("method_missing");

    DEFN(abs, 1, ch_abs);
    DEFN(sqrt, 1, ch_sqrt);
    DEFN(log, 1, ch_log);
    DEFN(log2, 1, ch_log2);
    DEFN(log10, 1, ch_log10);
    DEFN(sin, 1, ch_sin);
    DEFN(cos, 1, ch_cos);
    DEFN(tan, 1, ch_tan);
    DEFN(asin, 1, ch_asin);
    DEFN(acos, 1, ch_acos);
    DEFN(atan, 1, ch_atan);

    DEF(pi, ch_float(M_PI));
    DEF(e, ch_float(M_E));
}
