/* mem.h -- memory functions
   September 24, 2008

  Copyright (C) 2007-2008 Devin Papineau

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

  Devin Papineau jdmpapin@uwaterloo.ca
*/

#ifndef CH_MEM_H
#define CH_MEM_H

#include "value.h"
#include "misc.h"
#include <stddef.h>
#include <string.h>
#include <inttypes.h>

/* Boehm GC */
/* #define GC_DEBUG */
#include <gc/gc.h>

extern void ch_mem_init(uintptr_t*);

/**********
 * Tell the GC about a root that isn't on the stack.
 **********/
static inline void ch_globalRoot(void *ptr)
{
    /* do nothing */
}

/**********
 * Allocate some memory. The resulting memory is zeroed.
 **********/
static inline void *ch_malloc(size_t size)
{
    void *mem;

    mem = GC_MALLOC(size);
    if (!mem)
        fatal("out of memory");

    return mem;
}

/**********
 * Allocate memory that is guaranteed not to contain pointers.
 **********/
static inline void *ch_malloc_atomic(size_t size)
{
    void *mem;

    mem = GC_MALLOC_ATOMIC(size);
    if (!mem)
        fatal("out of memory");

    return mem;
}

/**********
 * Reallocate memory to have a different size.
 **********/
static inline void *ch_realloc(void *ptr, size_t size)
{
    void *mem;

    mem = GC_REALLOC(ptr, size);
    if (!mem)
        fatal("out of memory");

    return mem;
}

/**********
 * Set the finalizer for some memory.
 **********/
extern void ch_master_finalizer(void *ptr, void *fn);

static inline void ch_set_finalizer(void *ptr, void (*fn)(void*))
{
    if (!fn)
        return;

    void *dummy, *dummy2;
    GC_REGISTER_FINALIZER(ptr, ch_master_finalizer, (void*)fn,
                          (void(**)(void*,void*))&dummy, &dummy2);
}

/**********
 * Duplicate a string. The copy is in GCed memory.
 * (Is this used anywhere?)
 **********/
static inline char *ch_strdup(const char *s)
{
    size_t len;
    char* result;

    len = strlen(s);
    result = ch_malloc(len + 1);
    memcpy(result, s, len + 1);

    return result;
}

#endif /* CH_MEM_H */
