/* state.h -- extern vars for (hopefully most?) chai global state
   October 3, 2008

  Copyright (C) 2007-2008 Devin Papineau

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

  Devin Papineau jdmpapin@uwaterloo.ca
*/

#ifndef CH_STATE_H
#define CH_STATE_H

#include "value.h"
#include "mem.h"
#include "ht.h"

struct ch_obj;

typedef void (*ch_ctor_t)(struct ch_obj*);
typedef void (*ch_dupFn_t)(void*,void*);

typedef struct {
    int dataSize;
    ch_ctor_t ctor;
    ch_dupFn_t dup;
    ch_ctor_t dtor;
} ch_reprDataEntry_t;

typedef struct {
    ch_reprDataEntry_t* entries;
    int allocd;
    int nextID;
} ch_reprData_t;

extern ch_value_t ch_m_builtins, ch_m_root;

/* Search path */
extern ch_value_t ch_CHAIPATH;
extern char *ch_main_script_dir;
extern ch_value_t ch_argv;

/* Superclass of everything */
extern ch_value_t ch_cObject;

/* class Class, crazy! */
extern ch_value_t ch_cClass;

/* Some builtin types */
extern ch_value_t ch_cDict;
extern ch_value_t ch_cList;
extern ch_value_t ch_cString;

/* Function types */
extern ch_value_t ch_cFunction;  /* Superclass */
extern ch_value_t ch_cPrimitive;
extern ch_value_t ch_cClosure;

/* Immediate value types */
extern ch_value_t ch_cNilClass;
extern ch_value_t ch_cNumber;
extern ch_value_t ch_cFixnum;
extern ch_value_t ch_cFloat;
extern ch_value_t ch_cBoolean;
extern ch_value_t ch_cSymbol;

/* Represents a syntax tree */
extern ch_value_t ch_cASTNode;

/* File class */
extern ch_value_t ch_cFile;
/* Standard streams */
extern ch_value_t ch_stdin, ch_stdout, ch_stderr;

/* Bytecode class */
extern ch_value_t ch_cBytecode;

/* Classes for immediate values by type */
extern struct ch_obj* ch_immediateClasses[6];

/* Underlying object hidden data types */
extern ch_reprData_t ch_reprData;

/* Hash table of operator functions for the interpreter. */
extern ch_ht_t ch_opFuncs;

/* Where the parser puts the AST. */
extern ch_value_t ch_entireAST;

extern void ch_state_init(const char *program_name);

#endif /* CH_STATE_H */
