/* sym.c -- chai symbol implementation
   September 24, 2008

  Copyright (C) 2007-2008 Devin Papineau

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

  Devin Papineau jdmpapin@uwaterloo.ca
*/

#include "mem.h"
#include "value.h"
#include "sym.h"
#include "ht.h"
#include "Exception.h"
#include "sym_literals.h"
#include <string.h>
#include <stdio.h>

/* eq for the symbol table */
static int eqStr(ch_value_t a, ch_value_t b)
{
    return !strcmp(a.sym, b.sym);
}

/* Static symbol table, initialize on first use. */
static ch_ht_t* symTableHT = NULL;

void ch_symTable_init()
{
    ch_globalRoot(&symTableHT);
    symTableHT = ch_ht_create(ch_hashSym, eqStr);
    ch_symLiterals_init();
}

static inline ch_ht_t* symTable()
{
    return symTableHT;
}

/* cleanup function for symbols: removes it from the symbol table.
 * The symbol table doesn't hold a reference to the symbol (from a
 * refcounting perspective) so this can happen.
 */
#if 0 // TODO: remove
static void symbolCleanup(char* symText)
{
    ch_value_t keySym = { CH_SYMBOL };
    keySym.sym = symText;

//    printf("removing symbol `%s' from symbol table\n", symText);

    if (ch_ht_delete(symTable(), keySym) < 0)
        /* shouldn't ever happen */
        fatal("symbol exists that is not in symbol table: %s", symText);
}
#endif

/* Now we can actually intern things. */
ch_value_t ch_internCString(const char *str)
{
    return ch_internSizedString(strlen(str), str);
}

ch_value_t ch_internSizedString(size_t size, const char* str)
{
    ch_value_t key_sym, existing_sym;
    ch_ht_t* table;
    char* sym_text;

    key_sym.type = CH_SYMBOL;
    sym_text = (char*)ch_malloc_atomic(size + 1 + sizeof (long));
    memcpy(sym_text + sizeof (long), str, size);
    sym_text[size + sizeof (long)] = '\0';

    *((long*)sym_text) = ch_hashCString(str, size);
    key_sym.sym = sym_text;

    table = symTable();

    if (ch_ht_get(table, key_sym, &existing_sym) < 0) {
        /* It's not yet in the table. */
        ch_ht_set(table, key_sym, key_sym);
        return key_sym;
    }

    return existing_sym;
}

ch_value_t ch_gensym(const char *base)
{
    static unsigned int uniq = 0x4a;
    ch_value_t sym;
    char *text;

    /* hash value, %gs-, max 8 hex digits and terminating NUL */
    text = ch_malloc_atomic(strlen(base) + sizeof (long) + 13);
    sprintf(text + sizeof (long), "%%gs-%s%x", base, uniq++);

    *((long*)text) = ch_hashCString(text + sizeof (long), -1);

    sym.type = CH_SYMBOL;
    sym.sym = text;

    return sym;
}
