/* value.h -- chai dynamically typed values
   September 24, 2008

  Copyright (C) 2007-2008 Devin Papineau

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

  Devin Papineau jdmpapin@uwaterloo.ca
*/

#ifndef CH_VALUE_H
#define CH_VALUE_H

#include <stddef.h>

enum {
    CH_NIL = 0,
    CH_INT,
    CH_FLOAT,
    CH_BOOLEAN,
    CH_CMETHOD,
    CH_SYMBOL,
    CH_OBJECT,
    CH_REF
};

struct ch_value;
struct ch_obj;

typedef struct ch_value {
    long type;
    union {
        long i;
        float f;
        const char* sym;
        struct ch_obj* obj;
        struct ch_value* ref;
    };
} ch_value_t;

/* Canonical constants */
extern ch_value_t kNil, kFalse, kTrue;

static inline int ch_shallowEq(ch_value_t a, ch_value_t b)
{
    return a.type == b.type && a.i == b.i;
}

/* Hash functions. */
extern size_t ch_hashCString(const char* s, int len);

/* Hash function for the symbol table and object slot and class method
 * hashes. Prevents recursion on hashing a symbol. */
static inline size_t ch_hashSym(ch_value_t val)
{
    return *((long*)val.sym);
}

/* Make numbers */
static inline ch_value_t ch_fix(long x)
{
    ch_value_t result = { CH_INT };
    result.i = x;
    return result;
}
static inline ch_value_t ch_float(float x)
{
    ch_value_t result = { CH_FLOAT };
    result.f = x;
    return result;
}

#endif /* CH_VALUE_H */
